% Hint: turn cell mode on in Matlab to improve readability of this
% document. Go to Cell->Enable Cell Mode in the menu bar.

%% Simulate experiment and generate data
clear all
close all
clc

% Some administration
dt  = 0.01;                 % Sample time
T   = 50;                   % Total time
N   = T/dt;                 % number of samples
t   = (0:N-1).'*dt;         % Time vector

Fs  = 1./dt;                % Sample frequency
f   = (0:N-1).'/T;          % Frequency vector (double sided)

% Simulate model
options=simset('OutputPoints','specified'); % only produce output for the time values in vector <t>
sim('SISO_open_loop',t,options)
clear tout

% Plot input and output
figure
subplot(211), plot(t,u)
xlabel('time [s]')
ylabel('Input')
title ('Input and output signals')
subplot(212), plot(t,y)
ylabel('Output')
xlabel('time [s]')

%% Determine Fourier transforms and spectral densities
% Fourier transforms
U = fft(u);
Y = fft(y);

% Spectral densities (raw estimate)
Suu = 1/N * conj(U) .* U;
Suu = real(Suu);    % autospectral are real (imaginary part is negligible and results from the numerical operations 
Suy = 1/N * conj(U) .* Y;
Syu = 1/N * conj(Y) .* U;
Syy = 1/N * conj(Y) .* Y;
Syy = real(Syy);

%% Estimate transfer function and coherence
% Open loop transfer function Hol
% You will have to insert your own estimator and coherence function here.
% Keep the variable names Hol and Cohuy; these are used in plotting.

Hol = [];% !!! You need to insert your open loop estimator here.

% Squared coherence Cohuy function between u and y 
Cohuy = [];% !!! You need to insert your coherence function between u and y here

%% Apply frequency averaging and estimate transfer function
% Apply averaging
% The function freqAvg, which is provided, takes nband subsequent samples
% from a spectral density or frequency vector and averages them. As a
% demonstration run:
%       >> temp = 1:10
%       >> freqAvg(temp, 2)
%       >> freqAvg(temp, 4)
% in the command line.

Nband = 4; % Number of frequency bands to average over

mf   = freqAvg(f, Nband);   % Averaged frequency vector

mSuu = freqAvg(Suu, Nband); % Averaged spectral densities
mSuy = freqAvg(Suy, Nband);
mSyu = freqAvg(Syu, Nband);
mSyy = freqAvg(Syy, Nband);

% Estimate transfer function
mHol = []; % !!! You need to insert your averaged estimator here.

% Squared coherence Cohuy function between u and y 
mCohuy = [];% !!! You need to insert your averaged coherence function here.

if isempty(mHol) || isempty(mCohuy)
    error('Complete the m-file with your own estimators first!')
end

%% Apply averaging over segments (Welch) and estimate transfer function
% Apply averaging over segments
% The function welchspectrum, which is provided devides the data in
% Nsegment and calculates the sepctral density.

Nsegment = 4; % Number of segments to average over

wSuu = welchspectrum(u,u,Nsegment); % Averaged spectral densities
wSuu = real(wSuu);
wSuy = welchspectrum(y,u,Nsegment);
wSyu = welchspectrum(u,y,Nsegment);
wSyy = welchspectrum(y,y,Nsegment);
wSyy = real(wSyy);

Nd = length(wSuu);
Td = Nd*dt;
wf =(0:Nd-1).'/Td;   % Averaged frequency vector

% Estimate transfer function
wHol = []; % !!! You need to insert your averaged estimator here.

% Squared coherence Cohuy function between u and y 
wCohuy = [];% !!! You need to insert your averaged coherence function here.

if isempty(wHol) || isempty(wCohuy)
    error('Complete the m-file with your own estimators first!')
end


%% Determine true system transfer function
% Make transfer function
num = [1 0];
den = [0.03 0.001 1 0];
tftrue = tf(num, den);

% Determine transfer function values for each value in frequency vector f.
Htrue = squeeze(freqresp(tftrue, 2*pi*f));

%% Plot results of estimates and coherence
fmax = Fs/2; % Highest frequency in the plot

figure
subplot(311)
loglog(f, abs(Hol), 'b'); hold on
loglog(mf, abs(mHol), 'r');
loglog(wf, abs(wHol), 'm');
loglog(f, abs(Htrue), 'k--');
xlim([0.01 fmax])
ylabel('gain [-]')
title('Frequency response function and Coherence')

subplot(312)
semilogx(f, 180/pi*unwrap(angle(Hol)), 'b'); hold on
semilogx(mf, 180/pi*unwrap(angle(mHol)), 'r')
semilogx(wf, 180/pi*unwrap(angle(wHol)), 'm')
semilogx(f, 180/pi*unwrap(angle(Htrue)), 'k--')
ylim([-300 100])
xlim([0.01 fmax])
ylabel('phase [\circ]')

if ~isempty(Cohuy)
    subplot(313)
    semilogx(f, abs(Cohuy), 'b'); hold on
    if ~isempty(mCohuy)
        semilogx(mf, mCohuy, 'r');
    end
    if ~isempty(wCohuy)
        semilogx(wf, wCohuy, 'm');
    end
    ylim([0 1.5])
    xlim([0.01 fmax])
    ylabel('{\gamma_{yu}}^2 [-]')
    xlabel('frequency [Hz]')
end
